/*
 *              Copyright (c) 2016-2019, Nuralogix Corp.
 *                      All Rights reserved
 *
 *      THIS SOFTWARE IS LICENSED BY AND IS THE CONFIDENTIAL AND
 *      PROPRIETARY PROPERTY OF NURALOGIX CORP. IT IS
 *      PROTECTED UNDER THE COPYRIGHT LAWS OF THE USA, CANADA
 *      AND OTHER FOREIGN COUNTRIES. THIS SOFTWARE OR ANY
 *      PART THEREOF, SHALL NOT, WITHOUT THE PRIOR WRITTEN CONSENT
 *      OF NURALOGIX CORP, BE USED, COPIED, DISCLOSED,
 *      DECOMPILED, DISASSEMBLED, MODIFIED OR OTHERWISE TRANSFERRED
 *      EXCEPT IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF A
 *      NURALOGIX CORP SOFTWARE LICENSE AGREEMENT.
 */
#include "dfxcpp/CppChunkData.h"
#include "dfx/DFXErrors.h"

#include "dfxc/dfx_chunk_data.h"

using dfxcpp::CppChunkData;

using dfx::ThrowOnError;

CppChunkData::CppChunkData(std::unique_ptr<dfx_chunk_data, void(*)(dfx_chunk_data*)> chunkData) :
  dfxChunkData(std::move(chunkData)) {
}

CppChunkData::~CppChunkData() = default;

dfx::ChunkPayload CppChunkData::getChunkPayload() {
	dfx::ChunkPayload chunk;

	auto setValue = [&chunk](uint8_t valid, uint32_t startFrame, uint32_t endFrame, 
		uint32_t chunkNumber, uint32_t numberChunks, 
		uint32_t firstChunkStartTime_s,
		uint32_t startTime_s, uint32_t endTime_s,
		float duration_s, uint32_t numberPayloadBytes, uint8_t* payloadData,
		uint32_t numberMetadataBytes, uint8_t* metadata) {

		chunk.valid = valid;
		chunk.startFrame = startFrame;
		chunk.endFrame = endFrame;
		chunk.chunkNumber = chunkNumber;
		chunk.numberChunks = numberChunks;
		chunk.firstChunkStartTime_s = firstChunkStartTime_s;
		chunk.startTime_s = startTime_s,
		chunk.endTime_s = endTime_s;
		chunk.duration_s = duration_s;
		chunk.payload.insert(chunk.payload.end(), payloadData, payloadData+numberPayloadBytes);
		chunk.metadata.insert(chunk.metadata.end(), metadata, metadata+numberMetadataBytes);
	};
	using T = decltype(&setValue);
	auto callback = [](void* client_data,
		uint8_t  valid, 
		uint32_t startFrame, uint32_t endFrame, 
		uint32_t chunkNumber, uint32_t numberChunks,
		uint32_t firstChunkStartTime_s,
		uint32_t startTime_s, uint32_t endTime_s, float duration_s,
		uint32_t numberPayloadBytes, uint8_t* payloadData,
		uint32_t numberMetadataBytes, uint8_t* metadata) {

		auto fn = static_cast<T>(client_data);
		(*fn)(valid, startFrame, endFrame, 
			  chunkNumber, numberChunks,
			  firstChunkStartTime_s,  
			  startTime_s, endTime_s, duration_s, 
			  numberPayloadBytes, payloadData,
		      numberMetadataBytes, metadata);
	};

	dfx_chunk_data_get_chunk_payload(&(*dfxChunkData),
			callback, &setValue, ThrowOnError{});

	return chunk;
}
