/*
 *              Copyright (c) 2016-2019, Nuralogix Corp.
 *                      All Rights reserved
 *
 *      THIS SOFTWARE IS LICENSED BY AND IS THE CONFIDENTIAL AND
 *      PROPRIETARY PROPERTY OF NURALOGIX CORP. IT IS
 *      PROTECTED UNDER THE COPYRIGHT LAWS OF THE USA, CANADA
 *      AND OTHER FOREIGN COUNTRIES. THIS SOFTWARE OR ANY
 *      PART THEREOF, SHALL NOT, WITHOUT THE PRIOR WRITTEN CONSENT
 *      OF NURALOGIX CORP, BE USED, COPIED, DISCLOSED,
 *      DECOMPILED, DISASSEMBLED, MODIFIED OR OTHERWISE TRANSFERRED
 *      EXCEPT IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF A
 *      NURALOGIX CORP SOFTWARE LICENSE AGREEMENT.
 */
#include "dfxcpp/CppMeasurementData.h"

#include "dfx/DFXErrors.h"

using dfxcpp::CppMeasurementData;
using dfx::ThrowOnError;

CppMeasurementData::CppMeasurementData(
    std::unique_ptr<dfx_measurement_data, void(*)(dfx_measurement_data*)> measurementData) :
        dfxMeasurementData(std::move(measurementData))
{

}

CppMeasurementData::~CppMeasurementData() {
    // Nothing to do - cleared by unique_ptr
}

std::vector<std::string> CppMeasurementData::getDataPropertyKeys() {
    std::vector<std::string> keys;
    auto addResult = [&keys](const char* key) {
        keys.push_back(std::string(key));
    };
    using T = decltype(&addResult);
    auto callback = [](void* client_data, const char* name) {
        auto fn = static_cast<T>(client_data);
        (*fn)(name);
    };
    dfx_measurement_data_get_data_property_keys(&(*dfxMeasurementData), callback, &addResult, ThrowOnError{});
    return keys;
}

std::string CppMeasurementData::getDataProperty(const std::string& key) {
    std::string property;
    auto setValue = [&property](const char* prop) { property = prop; };
    using T = decltype(&setValue);
    auto callback = [](void* client_data, const char* prop) {
        auto fn = static_cast<T>(client_data);
        (*fn)(prop);
    };
    dfx_measurement_data_get_data_property(&(*dfxMeasurementData), key.c_str(), callback, &setValue, ThrowOnError{});
    return property;
}

std::vector<double> CppMeasurementData::getData() {
    std::vector<double> result;
    auto setValue = [&result](uint32_t data_length, double* data) {
        result.insert(result.end(), data, data+data_length);
    };
    using T = decltype(&setValue);
    auto callback = [](void* client_data, uint32_t data_length, double* data) {
        auto fn = static_cast<T>(client_data);
        (*fn)(data_length, data);
    };
    dfx_measurement_data_get_data(&(*dfxMeasurementData), callback, &setValue, ThrowOnError{});
    return result;
}
