/*
 *              Copyright (c) 2016-2019, Nuralogix Corp.
 *                      All Rights reserved
 *
 *      THIS SOFTWARE IS LICENSED BY AND IS THE CONFIDENTIAL AND
 *      PROPRIETARY PROPERTY OF NURALOGIX CORP. IT IS
 *      PROTECTED UNDER THE COPYRIGHT LAWS OF THE USA, CANADA
 *      AND OTHER FOREIGN COUNTRIES. THIS SOFTWARE OR ANY
 *      PART THEREOF, SHALL NOT, WITHOUT THE PRIOR WRITTEN CONSENT
 *      OF NURALOGIX CORP, BE USED, COPIED, DISCLOSED,
 *      DECOMPILED, DISASSEMBLED, MODIFIED OR OTHERWISE TRANSFERRED
 *      EXCEPT IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF A
 *      NURALOGIX CORP SOFTWARE LICENSE AGREEMENT.
 */
#include "dfxcpp/CppMeasurementResult.h"
#include "dfxcpp/CppMeasurementData.h"

#include "dfx/DFXErrors.h"

using dfxcpp::CppMeasurementResult;
using dfx::MeasurementData;
using dfx::ThrowOnError;

CppMeasurementResult::CppMeasurementResult(
    std::unique_ptr<dfx_measurement_result, void(*)(dfx_measurement_result*)> measurementResult) :
        dfxMeasurementResult(std::move(measurementResult))
{

}

CppMeasurementResult::~CppMeasurementResult() {
    // Nothing to do - cleared by unique_ptr
}

uint8_t CppMeasurementResult::isValid() {
    return dfx_measurement_result_is_valid(&(*dfxMeasurementResult), ThrowOnError{});
}

std::vector<std::string> CppMeasurementResult::getMeasurementPropertyKeys() {
    std::vector<std::string> keys;
    auto addResult = [&keys](const char* key) {
        keys.push_back(std::string(key));
    };
    using T = decltype(&addResult);
    auto callback = [](void* client_data, const char* name) {
        auto fn = static_cast<T>(client_data);
        (*fn)(name);
    };
    dfx_measurement_result_get_measurement_property_keys(&(*dfxMeasurementResult), callback, &addResult, ThrowOnError{});
    return keys;
}

std::string CppMeasurementResult::getMeasurementProperty(const std::string& key) {
    std::string property;
    auto setValue = [&property](const char* prop) { property = prop; };
    using T = decltype(&setValue);
    auto callback = [](void* client_data, const char* prop) {
        auto fn = static_cast<T>(client_data);
        (*fn)(prop);
    };
    dfx_measurement_result_get_measurement_property(&(*dfxMeasurementResult), key.c_str(), callback, &setValue, ThrowOnError{});
    return property;
}

std::vector<std::string> CppMeasurementResult::getMeasurementDataKeys() {
    std::vector<std::string> keys;
    auto addResult = [&keys](const char* key) {
        keys.push_back(std::string(key));
    };
    using T = decltype(&addResult);
    auto callback = [](void* client_data, const char* name) {
        auto fn = static_cast<T>(client_data);
        (*fn)(name);
    };
    dfx_measurement_result_get_measurement_data_keys(&(*dfxMeasurementResult), callback, &addResult, ThrowOnError{});
    return keys;
}

std::shared_ptr<dfx::MeasurementData> CppMeasurementResult::getMeasurementData(const std::string& key) {
    auto measurementData = dfx_measurement_result_get_measurement_data(&(*dfxMeasurementResult), key.c_str(), ThrowOnError{});
    if ( measurementData != nullptr ) {
        return std::make_shared<dfxcpp::CppMeasurementData>(
                std::unique_ptr<dfx_measurement_data, void(*)(dfx_measurement_data*)>(
                    measurementData, 
                    [](dfx_measurement_data* d) { dfx_measurement_data_destroy(d); }));
    }
    return nullptr; 
}

std::string CppMeasurementResult::getErrorCode() {
    std::string errorCode;
    auto setValue = [&errorCode](const char* error) { errorCode = error; };
    using T = decltype(&setValue);
    auto callback = [](void* client_data, const char* error) {
        auto fn = static_cast<T>(client_data);
        (*fn)(error);
    };
    dfx_measurement_result_get_error_code(&(*dfxMeasurementResult), callback, &setValue, ThrowOnError{});
    return errorCode;
}
