/*
 *              Copyright (c) 2016-2019, Nuralogix Corp.
 *                      All Rights reserved
 *
 *      THIS SOFTWARE IS LICENSED BY AND IS THE CONFIDENTIAL AND
 *      PROPRIETARY PROPERTY OF NURALOGIX CORP. IT IS
 *      PROTECTED UNDER THE COPYRIGHT LAWS OF THE USA, CANADA
 *      AND OTHER FOREIGN COUNTRIES. THIS SOFTWARE OR ANY
 *      PART THEREOF, SHALL NOT, WITHOUT THE PRIOR WRITTEN CONSENT
 *      OF NURALOGIX CORP, BE USED, COPIED, DISCLOSED,
 *      DECOMPILED, DISASSEMBLED, MODIFIED OR OTHERWISE TRANSFERRED
 *      EXCEPT IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF A
 *      NURALOGIX CORP SOFTWARE LICENSE AGREEMENT.
 */
#include "dfxcpp/DFXFactory.h"
#include "dfxcpp/CppCollector.h"
#include "dfxcpp/CppFrame.h"

#include "dfx/DFXErrors.h"

#include "dfxc/dfx_factory.h"

using dfxcpp::DFXFactory;
using dfx::ThrowOnError;

DFXFactory::DFXFactory() :
	dfx::DFXFactory(), 
	dfxFactory(
		std::unique_ptr<dfx_factory, void(*)(dfx_factory*)>(
			dfx_factory_create(),
			[](dfx_factory* f){ dfx_factory_destroy(f); })) 	
{	
}

DFXFactory::~DFXFactory() {
}

std::string DFXFactory::getVersion() const {
	return dfx_factory_get_version(&(*dfxFactory), ThrowOnError{});
}

std::string DFXFactory::getSdkID() const {
    return dfx_factory_get_sdk_id(&(*dfxFactory), ThrowOnError{});
}

bool DFXFactory::setMode(const std::string& mode) {
	return dfx_factory_set_mode(&(*dfxFactory), mode.c_str(), ThrowOnError{});
}

std::string DFXFactory::getMode() const {
	return dfx_factory_get_mode(&(*dfxFactory), ThrowOnError{});
}

std::string DFXFactory::getLastErrorMessage() {
	std::string lastErrorMessage;
	auto setValue = [&lastErrorMessage](const char* message) { lastErrorMessage = message; };
	using T = decltype(&setValue);
	auto callback = [](void* client_data, const char* message) {
		auto fn = static_cast<T>(client_data);
		(*fn)(message);
	};
	dfx_factory_get_last_error_message(&(*dfxFactory), callback, &setValue, ThrowOnError{});
	return lastErrorMessage;
}

void DFXFactory::setProperty(const std::string &key, const std::string &value) {
	dfx_factory_set_property(&(*dfxFactory), key.c_str(), value.c_str(), ThrowOnError{});
}

std::string DFXFactory::getProperty(const std::string &key) {
	std::string property;
	auto setValue = [&property](const char* prop) { property = prop; };
	using T = decltype(&setValue);
	auto callback = [](void* client_data, const char* prop) {
		auto fn = static_cast<T>(client_data);
		(*fn)(prop);
	};
	dfx_factory_get_property(&(*dfxFactory), key.c_str(), callback, &setValue, ThrowOnError{});
	return property;
}

void DFXFactory::removeProperty(const std::string &key) {
	dfx_factory_remove_property(&(*dfxFactory), key.c_str(), ThrowOnError{});
}

std::vector<std::string> DFXFactory::getValidProperties() const {
	std::vector<std::string> props;
    auto addResult = [&props](const char* name) {
        props.push_back(std::string(name));
    };
	using T = decltype(&addResult);
    auto callback = [](void* client_data, const char* name) {
        auto fn = static_cast<T>(client_data);
        (*fn)(name);
    };
    dfx_factory_get_valid_properties(&(*dfxFactory), callback, &addResult, ThrowOnError{});
	return props;
}

bool DFXFactory::initializeStudy(const std::vector<unsigned char> &data) {
	bool result = dfx_factory_initialize_study(&(*dfxFactory),
		data.size(), data.data(), ThrowOnError{});
	return result;
}

bool DFXFactory::addToStudy(const std::vector<unsigned char> &data) {
	bool result = dfx_factory_add_to_study(&(*dfxFactory),
		data.size(), data.data(), ThrowOnError{});
	return result;
}

bool DFXFactory::initializeStudyFromFile(const std::string &pathToFile) {
	bool result = dfx_factory_initialize_study_from_file(&(*dfxFactory),
		pathToFile.c_str(), ThrowOnError{});
	return result;
}

bool DFXFactory::addToStudyFromFile(const std::string &pathToFile) {
	bool result = dfx_factory_add_to_study_from_file(&(*dfxFactory),
		pathToFile.c_str(), ThrowOnError{});
	return result;
}

std::shared_ptr<dfx::Collector> DFXFactory::createCollector() {
	auto dfxCollector = dfx_collector_create(&(*dfxFactory));
	return std::make_shared<dfxcpp::CppCollector>(
		std::unique_ptr<dfx_collector, void(*)(dfx_collector*)>(dfxCollector,
			[](dfx_collector* c){ dfx_collector_destroy(c); }));
}

